@@ -22,5 +22,7 @@ stock_recursive_download = False
 stock_not_interface = False
 stock_zip = False
 stock_real_time_dl = False
+# New: default real-time multiplier (0-10). 1 means real-time, 0 disables pacing.
+stock_real_time_multiplier = 1
 stock_save_cover = False # Default for saving cover image
 stock_market = None
