@@ -135,7 +135,7 @@ def what_kind(link):
 def __get_tronc(string):
     return string[:len(string) - 1]
 
-def apply_custom_format(format_str, metadata: dict, pad_tracks=True) -> str:
+def apply_custom_format(format_str, metadata: dict, pad_tracks=True, pad_number_width: int | None = None) -> str:
     def replacer(match):
         full_key = match.group(1)  # e.g., "artist", "ar_album_1"
 
@@ -183,7 +183,7 @@ def apply_custom_format(format_str, metadata: dict, pad_tracks=True) -> str:
 
             # Handle None values safely
             if value is None:
-                if full_key in ['tracknum', 'discnum']:
+                if full_key in ['tracknum', 'discnum', 'playlistnum']:
                     value = '1' if full_key == 'discnum' else '0'
                 else:
                     value = ''
@@ -196,14 +196,18 @@ def apply_custom_format(format_str, metadata: dict, pad_tracks=True) -> str:
 
             if pad_tracks and full_key in ['tracknum', 'discnum', 'playlistnum']:
                 str_value = str(value)
-                # Pad with leading zero if it's a single digit
-                if str_value.isdigit() and len(str_value) == 1:
-                    return str_value.zfill(2)
+                if str_value.isdigit():
+                    if isinstance(pad_number_width, int) and pad_number_width >= 1:
+                        return str_value.zfill(pad_number_width)
+                    # Default legacy behavior: pad single digits to width 2
+                    if len(str_value) == 1:
+                        return str_value.zfill(2)
+                return str_value
             return str(value)
 
     return re.sub(r'%([^%]+)%', replacer, format_str)
 
-def __get_dir(song_metadata, output_dir, custom_dir_format=None, pad_tracks=True):
+def __get_dir(song_metadata, output_dir, custom_dir_format=None, pad_tracks=True, pad_number_width: int | None = None):
     # If custom_dir_format is explicitly empty or None, use output_dir directly
     if not custom_dir_format:
         # Ensure output_dir itself exists, as __check_dir won't be called on a subpath
@@ -214,7 +218,7 @@ def __get_dir(song_metadata, output_dir, custom_dir_format=None, pad_tracks=True
     # create directories; slashes from data are sanitized inside components.
     format_parts = custom_dir_format.split("/")
     formatted_parts = [
-        apply_custom_format(part, song_metadata, pad_tracks) for part in format_parts
+        apply_custom_format(part, song_metadata, pad_tracks, pad_number_width) for part in format_parts
     ]
     sanitized_path_segment = "/".join(
         sanitize_name(part) for part in formatted_parts
@@ -233,14 +237,16 @@ def set_path(
     is_episode=False,
     custom_dir_format=None,
     custom_track_format=None,
-    pad_tracks=True
+    pad_tracks=True,
+    pad_number_width: int | None = None
 ):
     # Determine the directory for the song
     directory = __get_dir(
         song_metadata,
         output_dir,
         custom_dir_format=custom_dir_format,
-        pad_tracks=pad_tracks
+        pad_tracks=pad_tracks,
+        pad_number_width=pad_number_width
     )
 
     # Determine the filename for the song
@@ -262,7 +268,7 @@ def set_path(
 
     # Apply the custom format string for the track filename.
     # pad_tracks is passed along for track/disc numbers in filename.
-    track_filename_base = apply_custom_format(custom_track_format, effective_metadata, pad_tracks)
+    track_filename_base = apply_custom_format(custom_track_format, effective_metadata, pad_tracks, pad_number_width)
     track_filename_base = sanitize_name(track_filename_base)
 
     # Add file format (extension) to the filename
