@@ -2,6 +2,7 @@
 
 from dataclasses import dataclass
 from typing import Optional
+from enum import Enum
 
 
 @dataclass
@@ -18,4 +19,10 @@ class ReleaseDate:
     """Mandatory release date structure."""
     year: int
     month: Optional[int] = None  # null if unknown
-    day: Optional[int] = None    # null if unknown 
\ No newline at end of file
+    day: Optional[int] = None    # null if unknown
+
+
+class Service(str, Enum):
+    """Supported source services for downloads."""
+    SPOTIFY = "spotify"
+    DEEZER = "deezer" 
\ No newline at end of file
