@@ -17,6 +17,8 @@ from os import (
     system,
     replace as os_replace,
 )
+import subprocess
+import shutil
 from deezspot.models.download import (
     Track,
     Album,
@@ -60,6 +62,7 @@ from deezspot.models.callback import (
     IDs
 )
 from deezspot.spotloader.__spo_api__ import tracking, json_to_track_playlist_object
+from deezspot.models.callback.common import Service
 
 # --- Global retry counter variables ---
 GLOBAL_RETRY_COUNT = 0
@@ -144,6 +147,25 @@ class EASY_DW:
         pad_tracks = getattr(self.__preferences, 'pad_tracks', True)
         # Ensure the separator is available to formatting utils for indexed placeholders
         self.__song_metadata_dict['artist_separator'] = getattr(self.__preferences, 'artist_separator', '; ')
+        # Determine pad width (supports 'auto' mode)
+        pad_number_width = None
+        try:
+            pnw = getattr(self.__preferences, 'pad_number_width', None)
+            if isinstance(pnw, str) and pnw.lower() == 'auto':
+                total = None
+                if self.__parent == 'album' and hasattr(self.__song_metadata, 'album') and getattr(self.__song_metadata.album, 'total_tracks', None):
+                    total = self.__song_metadata.album.total_tracks
+                elif self.__parent == 'playlist' and hasattr(self.__preferences, 'json_data') and self.__preferences.json_data:
+                    try:
+                        total = self.__preferences.json_data.get('tracks', {}).get('total')
+                    except Exception:
+                        total = None
+                if isinstance(total, int) and total and total > 0:
+                    pad_number_width = max(2, len(str(total)))
+            elif isinstance(pnw, int) and pnw >= 1:
+                pad_number_width = pnw
+        except Exception:
+            pad_number_width = None
         # Inject playlist placeholders if in playlist context
         try:
             if self.__parent == 'playlist' and hasattr(self.__preferences, 'json_data') and self.__preferences.json_data:
@@ -165,7 +187,8 @@ class EASY_DW:
             self.__file_format,
             custom_dir_format=custom_dir_format,
             custom_track_format=custom_track_format,
-            pad_tracks=pad_tracks
+            pad_tracks=pad_tracks,
+            pad_number_width=pad_number_width
         )
 
     def __set_episode_path(self) -> None:
@@ -242,8 +265,23 @@ class EASY_DW:
         try:
             # Step 1: First convert the OGG file to standard format (copy operation)
             # Output is og_song_path_for_ogg_output
-            ffmpeg_cmd = f'ffmpeg -y -hide_banner -loglevel error -i "{temp_filename}" -c:a copy "{og_song_path_for_ogg_output}"'
-            system(ffmpeg_cmd) # Creates/overwrites og_song_path_for_ogg_output
+            # Resolve ffmpeg path explicitly to avoid PATH issues in distroless
+            ffmpeg_path = shutil.which("ffmpeg") or "/usr/local/bin/ffmpeg"
+            try:
+                result = subprocess.run(
+                    [
+                        ffmpeg_path, "-y", "-hide_banner", "-loglevel", "error",
+                        "-i", temp_filename, "-c:a", "copy", og_song_path_for_ogg_output
+                    ],
+                    stdout=subprocess.PIPE,
+                    stderr=subprocess.PIPE,
+                    text=True,
+                    check=False
+                )
+                if result.returncode != 0 or not os.path.exists(og_song_path_for_ogg_output):
+                    raise RuntimeError(f"ffmpeg remux failed (rc={result.returncode}). stderr: {result.stderr.strip()}")
+            except FileNotFoundError as fnf:
+                raise RuntimeError(f"ffmpeg not found: attempted '{ffmpeg_path}'. Ensure it is present in PATH.") from fnf
             
             # temp_filename has been processed. Unregister and remove it.
             # CURRENT_DOWNLOAD was temp_filename.
@@ -701,7 +739,8 @@ class EASY_DW:
                 failed_tracks=[],
                 total_successful=len(successful_track_list),
                 total_skipped=len(skipped_track_list),
-                total_failed=0
+                total_failed=0,
+                service=Service.SPOTIFY
             )
             # Enrich summary with final path and quality
             try:
@@ -716,6 +755,28 @@ class EASY_DW:
             }
             summary_obj.final_path = final_path_val
             summary_obj.download_quality = sp_quality_map_single.get(quality_key_single, 'OGG')
+            # Compute final quality/bitrate
+            quality_val = None
+            bitrate_val = None
+            if self.__convert_to:
+                # When converting, trust convert_to + bitrate
+                fmt = self.__convert_to
+                if fmt:
+                    quality_val = fmt.lower()
+                br_raw = self.__bitrate
+                if br_raw:
+                    digits = ''.join([c for c in str(br_raw) if c.isdigit()])
+                    bitrate_val = f"{digits}k" if digits else None
+            else:
+                quality_val = 'ogg'
+                if quality_key_single == 'NORMAL':
+                    bitrate_val = '96k'
+                elif quality_key_single == 'HIGH':
+                    bitrate_val = '160k'
+                elif quality_key_single == 'VERY_HIGH':
+                    bitrate_val = '320k'
+            summary_obj.quality = quality_val
+            summary_obj.bitrate = bitrate_val
 
         # Report track done status
         # Compute final path and quality label
@@ -980,16 +1041,27 @@ def download_cli(preferences: Preferences) -> None:
     __not_interface = preferences.not_interface
     __quality_download = preferences.quality_download
     __recursive_download = preferences.recursive_download
-    cmd = f"deez-dw.py -so spo -l \"{__link}\" "
+    # Build argv list instead of shell string (distroless-safe)
+    argv = ["deez-dw.py", "-so", "spo", "-l", __link]
     if __output_dir:
-        cmd += f"-o {__output_dir} "
+        argv += ["-o", str(__output_dir)]
     if __not_interface:
-        cmd += f"-g "
+        argv += ["-g"]
     if __quality_download:
-        cmd += f"-q {__quality_download} "
+        argv += ["-q", str(__quality_download)]
     if __recursive_download:
-        cmd += f"-rd "
-    system(cmd)
+        argv += ["-rd"]
+    prog = shutil.which(argv[0])
+    if not prog:
+        logger.error("deez-dw.py CLI not found in PATH; cannot run download_cli in this environment.")
+        return
+    argv[0] = prog
+    try:
+        result = subprocess.run(argv, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
+        if result.returncode != 0:
+            logger.error(f"deez-dw.py exited with {result.returncode}: {result.stderr.strip()}")
+    except Exception as e:
+        logger.error(f"Failed to execute deez-dw.py: {e}")
 
 class DW_TRACK:
     def __init__(
@@ -1064,6 +1136,7 @@ class DW_ALBUM:
                 c_preferences.link = f"https://open.spotify.com/track/{c_preferences.ids}"
                 # Set album position for progress reporting (not for metadata - that comes from API)
                 c_preferences.track_number = a + 1
+                c_preferences.pad_number_width = getattr(self.__preferences, 'pad_number_width', 'auto')
 
                 track = EASY_DW(c_preferences, parent='album').easy_dw()
 
@@ -1118,8 +1191,30 @@ class DW_ALBUM:
             failed_tracks=failed_tracks,
             total_successful=len(successful_tracks),
             total_skipped=len(skipped_tracks),
-            total_failed=len(failed_tracks)
+            total_failed=len(failed_tracks),
+            service=Service.SPOTIFY
         )
+        # Compute final quality/bitrate for album summary
+        quality_val = None
+        bitrate_val = None
+        conv = getattr(self.__preferences, 'convert_to', None)
+        if conv:
+            quality_val = conv.lower()
+            br_raw = getattr(self.__preferences, 'bitrate', None)
+            if br_raw:
+                digits = ''.join([c for c in str(br_raw) if c.isdigit()])
+                bitrate_val = f"{digits}k" if digits else None
+        else:
+            quality_val = 'ogg'
+            qkey = (getattr(self.__preferences, 'quality_download', None) or 'NORMAL').upper()
+            if qkey == 'NORMAL':
+                bitrate_val = '96k'
+            elif qkey == 'HIGH':
+                bitrate_val = '160k'
+            elif qkey == 'VERY_HIGH':
+                bitrate_val = '320k'
+        summary_obj.quality = quality_val
+        summary_obj.bitrate = bitrate_val
         
         report_album_done(album_obj, summary_obj)
         
@@ -1195,6 +1290,7 @@ class DW_PLAYLIST:
             c_preferences.json_data = self.__json_data
             c_preferences.track_number = idx + 1
             c_preferences.link = f"https://open.spotify.com/track/{c_preferences.ids}" if c_preferences.ids else None
+            c_preferences.pad_number_width = getattr(self.__preferences, 'pad_number_width', 'auto')
 
             easy_dw_instance = EASY_DW(c_preferences, parent='playlist')
 
@@ -1250,8 +1346,30 @@ class DW_PLAYLIST:
             failed_tracks=failed_tracks_cb,
             total_successful=len(successful_tracks_cb),
             total_skipped=len(skipped_tracks_cb),
-            total_failed=len(failed_tracks_cb)
+            total_failed=len(failed_tracks_cb),
+            service=Service.SPOTIFY
         )
+        # Compute final quality/bitrate for playlist summary
+        quality_val = None
+        bitrate_val = None
+        conv = getattr(self.__preferences, 'convert_to', None)
+        if conv:
+            quality_val = conv.lower()
+            br_raw = getattr(self.__preferences, 'bitrate', None)
+            if br_raw:
+                digits = ''.join([c for c in str(br_raw) if c.isdigit()])
+                bitrate_val = f"{digits}k" if digits else None
+        else:
+            quality_val = 'ogg'
+            qkey = (getattr(self.__preferences, 'quality_download', None) or 'NORMAL').upper()
+            if qkey == 'NORMAL':
+                bitrate_val = '96k'
+            elif qkey == 'HIGH':
+                bitrate_val = '160k'
+            elif qkey == 'VERY_HIGH':
+                bitrate_val = '320k'
+        summary_obj.quality = quality_val
+        summary_obj.bitrate = bitrate_val
         
         # Include m3u path in summary and callback
         report_playlist_done(playlist_obj_for_cb, summary_obj, m3u_path=m3u_path)
