@@ -25,4 +25,8 @@ qualities = {
 	}
 }
 
+# Document the allowed bounds for real-time pacing speed-ups
+REAL_TIME_MULTIPLIER_MIN = 0
+REAL_TIME_MULTIPLIER_MAX = 10
+
 stock_market = None
